/*lcdprint  v0.2.1            (c)2001 by Andreas Schwarz (andreas-s@web.de)
  minor upgrades			   Volker Urban

 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#include "lcd_lib.c"


int do_init=0, do_set=0, do_new=0, do_clear=0, do_home=0, do_shift=0;
int do_set_datawidth=0, do_lcd_modes=0,do_print_char=0,do_load_char=0;
int do_test=0;
char print_char=0xa0; /*space*/
int test_low_width = 1;
int initdelay = 4 ;
unsigned char go_x, go_y;

int position=0;

const char USAGE[] =
"lcdprint  v0.2.1 (9.02.2002) (c)2001 by Andreas Schwarz (andreas-s@web.de)\n"
"with minor upgrades by Volker Urban (volker.urban@web.de)\n"
"Prints on a 2x16 LCD connected to parallel port\n\n"
"Usage: lcd.exe [options] text...\n"
"Options:\n"
"  -p[278|378|3bc] Parallelport at address 0x278 (or x378 or x3bc),\n"
"                   default 378\n"
"  -w[4|8]         Use 4 (or 8) bit LCD interface, default 8\n"
"  -i[basedelay]   Init LCD, basedelay shall last 150 ns\n"
"  -s [char line]  Set cursor position\n"
"  -c              Clear Display\n"
"  -h              Cursor Home\n"
"  -f[cr]          Shift cursor right,\n"
"					default: (w/o c or r): shift line left\n"
"  -m[DCB]         Set Display and Cursor and cursorBlinking to ON ON ON\n"
"				   default: (w/o D C or B) off\n"
"  -a[code]        Show character with code number code\n"
"  -u              load Character generator RAM\n"
"  -t[delay]       test port, means toggling every pin every delay/2*delay\n"
"					(33% duty cycle); delay=0 sets noninverting pins to '1'\n"
"  -?              This usage.\n\n";

/* char pyramchar[]={0x01,0x03,0x07,0x0f,0x1f,0x1e,0x1c};
char pyramchar[]={0x1f,0x0f,0x07,0x03,0x01,0x1f,0x1f};*/
char pyramchar[]={0x03,0x07,0x0f,0x1f,0x0f,0x07,0x03};

int ShowUsage( char *szError ) {

  fprintf( stderr, USAGE );

  fprintf(stderr, szError);

  return 0;
}


int main( int argc, char *argv[] )
{
   int i;

   if (argc == 1) {
	  return ShowUsage("");
   }

   for (i=1; i<argc; i++)
	{

	  if (argv[i][0] != '-') {
		 position = i;
		 break;
	  }

	  switch ( argv[i][1] )
		{
		  case 'i':
			do_init=1;
			initdelay=atoi(argv[i]+2);
			break;

		  case 's':
			if ((i+2 >= argc)) //mindestens 2 nachfolgende Parameter?
			   return ShowUsage("Error: Missing parameter for -s\n");
			if (argv[i+1][1]!=0) {
			   go_x = 10 * (argv[i+1][0] - 48) + (argv[i+1][1] - 48); //wenn x zweistellig ist
			} else {
			   go_x = argv[i+1][0] - 48; //wenn x einstellig ist
			}
			go_y = argv[i+2][0] - 48; //y muss immer einstellig sein
			i+=2; //die 2 nachfolgenden Parameter mssen nicht mehr verarbeitet werden
			do_set=1;
			break;

		  case 'p':
		   if (strcmp(argv[i]+2,"278\0")==0)
			LCD_Data_port = 0x278;
		   else if (strcmp(argv[i]+2,"378\0")==0)
			LCD_Data_port =0x378;
		   else if (strcmp(argv[i]+2,"3bc\0")==0)
			LCD_Data_port =0x3bc;

		   LCD_Control_port = LCD_Data_port+2;
		   break;

		  case 'c':
			do_clear=1;
			break;

		 case 'u':
		   do_load_char=1;
		   break;

		 case 'f':
			do_shift=0;
			if ((argv[i][2]=='r')|(argv[i][3]=='r'))
			  do_shift=2; else do_shift=1;
			if ((argv[i][2]!='c')&(argv[i][3]!='c'))
			  do_shift+=2;
			break;

		 case 'm':
			do_lcd_modes=0;{
			  int a=2; while (argv[i][a] != '\0') {
				switch (argv[i][a]){
				  case 'D': do_lcd_modes+=4;break;
				  case 'C': do_lcd_modes+=2;break;
				  case 'B': do_lcd_modes++;break;
				}
				a++;
			  }
			 }
			do_lcd_modes++;break;

		  case 'h':
			do_home=1;
			break;

		  case '?':
			return ShowUsage("");
		   /*	break; */

		  case 'w': /* datawidth of lcd interface (usally 4|8 bit) */
		   do_set_datawidth=atoi(argv[i]+2);
		   break;

		  case 'a': /*show character*/
		   do_print_char=1;
		   print_char= (char) atoi(argv[i]+2);
		   break;

		  case 't': /* send pulses to every portbit */
		   do_test=1;
		   test_low_width = atoi(argv[i]+2);
		   printf("Endless loop. Any key to exit\n");
		   break;

		  default:
			return ShowUsage("Error: Invalid option\n");
		}
   }

   if (do_set_datawidth!=0)
	  lcd_set_datawidth(do_set_datawidth);

   if(do_init==1) {
	  lcd_init(initdelay);
	  waitms();
   }

   if(do_clear==1) {
	  lcd_clrscr();
   waitms();
   }

   if (do_lcd_modes!=0){
	 do_lcd_modes--;
	 lcd_modes(do_lcd_modes/4,(do_lcd_modes&0x2)>>1,do_lcd_modes%2);
   }

   if(do_home==1) {
	  lcd_home();
	  waitms();
   }

   if(do_shift!=0) {
	  switch (do_shift) {
		case 4: lcd_shift(1,1); break;
		case 3: lcd_shift(1,0); break;
		case 2: lcd_shift(0,1); break;
		case 1: lcd_shift(0,0); break;
		default: break;
	  }

	  waitms();
   }


   if(do_load_char!=0){
	  lcd_load_character(0,7,pyramchar);
	  /*lcd_clrscr();  */
   }


   if(do_print_char!=0)
	lcd_putchar(print_char);

   if(do_set==1) {
	  lcd_gotoxy(go_x-1, go_y-1);
	  waitms();
   }

   if(do_test!=0)
	port_test(test_low_width);

   if (position!=0) {
	  for(i=position; i<argc; i++) {
		 lcd_puts(argv[i]);
		 if (i+1<argc)
			lcd_putchar(' ');
	  }
   }

	return 0;
}
