#ifndef	LCDLIB_C
#define LCDLIB

#include <CONIO.H>
#include <DOS.H>



void lcd_write_command8(int value);
void lcd_write_data8(int value);
void lcd_init8(int basedelay);

void lcd_write_command4(int value);
void lcd_write_data4(int value);
void lcd_init4(int basedelay);

void lcd_shift(int entire_display, int right);
void lcd_load_character(int char_pos,int char_lines,char *char_pattern);
void lcd_modes(int ONoff, int ONcursor, int ONblink);
void lcd_goto(void);
void lcd_gotoxy(int x, int y);
void lcd_newline(void);
void lcd_clrscr(void);
void lcd_home(void);
void lcd_putchar(char data);
void lcd_puts(char s[]);

void waitms(void);
void port_test(int low_width);

void (*lcd_write_command)(int)=lcd_write_command8;
void (*lcd_write_data)(int)=lcd_write_data8;
void (*lcd_init)(int)=lcd_init8;

static int  lcd_x, lcd_y;

/*Which Port is connected with LCD (means which Port has Parallel Port*/

unsigned int LCD_Data_port    =  0x378;
unsigned int LCD_Control_port =  0x37a;

int basedelay = 4; /* length of counting loop, lasting 150 ns */
/*Remember bit 3,1-0 in Port Control are inverted;
 bit 0 LCD enable;
 bit 1 LCD RS */

/* 8 bit interface */

#pragma warn -eff /* prevent "Code has no effect warning" while outp()*/

void lcd_set_datawidth(int width){
		if (width==4) {
		  lcd_write_data=lcd_write_data4;
		  lcd_write_command=lcd_write_data4;
		  lcd_init=lcd_init4;}
		else {
		  lcd_write_data=lcd_write_data8;
		  lcd_write_command=lcd_write_data8;
		  lcd_init=lcd_init8;}
}

/* turn optimisation off, cause of timing built by counting empty loop */
void lcd_write_command8(int value) {
		int i;
		outp(LCD_Data_port, value);
		for(i=basedelay;i!=0;i--);  /* 140 ns setup */
		outp(LCD_Control_port, 0x02); /* 450 ns pulsewidth*/
		for(i=3*basedelay;i!=0;i--);  /* 450 ns  */
		outp(LCD_Control_port, 0x03); /* t cycle 1000 ns */
		waitms();
}

void lcd_write_data8(int value) {
		int i;
		outp(LCD_Data_port, value);
		for(i=basedelay;i!=0;i--);  /* 140 ns setup */
		outp(LCD_Control_port, 0x00);
		for(i=3*basedelay;i!=0;i--);  /* 450 ns pulse width */
		outp(LCD_Control_port, 0x01);
		waitms();
}

#pragma warn +eff

void lcd_init8(int delay)
{
		if (delay==0) delay = 4;
		basedelay=delay;
		lcd_write_command8(0x30); /*8bit interface, 2 lines, fontsize */
		{int i;for(i=5;i!=0;i--) waitms();}
		lcd_write_command8(0x30); /*clear_display*/
		waitms();
		lcd_write_command8(0x30); /*set function*/
		lcd_write_command8(0x38); /*set function*/
		lcd_write_command8(0x08); /* display off*/
		lcd_write_command8(0x01); /*clear_display*/
		lcd_write_command8(0x06); /*increment cursor, no display shift */
		lcd_write_command8(0x0C); /*Display on, cursor and blink off */
}

/* 4 bit interface */

void lcd_write_command4(int value) {

		lcd_write_command8(value&0xf0);
		lcd_write_command8(value<<4);
}

void lcd_write_data4(int value) {
		lcd_write_data8(value&0xf0);
		lcd_write_data8(value<<4);
}

void lcd_init4(int delay)
{
		if (delay==0) delay = 4;
		basedelay=delay;
		lcd_write_command8(0x30); /*8bit interface, 2 lines, fontsize */
		lcd_write_command8(0x30);  /* */
		{int i;for(i=5;i!=0;i--) waitms();}
		lcd_write_command8(0x30);
		waitms();
		lcd_write_command8(0x30);
		lcd_write_command8(0x20);
		lcd_write_command4(0x28); /*4bit interface, 2 lines, fontsize */
		lcd_write_command4(0x08); /*Display OFF */

		lcd_write_command4(0x01); /*clear_display*/
		waitms();
		lcd_write_command4(0x06); /*increment cursor, no display shift */
		lcd_write_command4(0x0C); /*Display on, cursor and blink off */
}

 /* "High Level LCD Operations"*/

void lcd_goto(void)
/* goto position (lcd_x,lcd_y) */
{
	lcd_write_command((1<<7)+0x40*lcd_y+lcd_x);
}

void lcd_gotoxy(int x, int y)
/* goto position (x,y) */
{
	lcd_x = x; lcd_y = y;
	lcd_goto();
}

void lcd_newline(void)
/* goto start of next line */
{
	lcd_x = 0;
	if (++lcd_y >= 2)
		lcd_y = 0;
}

void lcd_clrscr(void)
/* clear lcd */
{
	lcd_x = lcd_y = 0;
	lcd_write_command(1<<0);
}

void lcd_home(void)
/* set cursor to home position */
{
	lcd_x = lcd_y = 0;
	lcd_write_command(1<<1);
}

void lcd_putchar(char data)
/* print character to current cursor position */
{
	if (data=='\n') {
		lcd_newline();
		lcd_goto();
	}
	else {
		if (++lcd_x >= 17) {
			lcd_newline();
			lcd_goto();
		}
		lcd_write_data(data);
	}
}

void lcd_puts(char s[])
/* print string on lcd (no auto linefeed) */
{
	char *c;

	if (!(c=s)) return;

	while (*c) {
		lcd_putchar(*c);
		c++;
	}
}

void lcd_shift(int entire_display, int right)
/* shift display or cursor */
{
	lcd_write_command(0x10+entire_display*8+4*right);
}

void lcd_modes(int ONoff, int ONcursor, int ONblink){
/* switch between display,cursor,cursor blink ONOFF*/
	lcd_write_command(0x8+4*ONoff+2*ONcursor+ONblink);
}

void lcd_load_character(int char_position, int char_lines,char *char_pattern)
/* load character to caracter generator RAM*/
{
	int i;

	lcd_write_command(0x40+(char_position<<3));  /* set CG RAM address */
	waitms();
	for (i=char_lines;i!=0;i--){
		lcd_write_data(*char_pattern++);
		waitms();
	}
	lcd_write_data(0x00); /*cursorline*/
	lcd_write_command(0x80); /* set DDRAM adress, for reseting AC*/
	waitms();

}


/* misc */

void waitms(void) {
   delay(1);
}

/* set every 2nd bit to '0'; others to '1'
   negate all bits every 0.5 secs
   endless loop
*/
#pragma warn -eff
void port_test(int low_width) {
		while (1){
			outp(LCD_Data_port, 0xff);
			outp(LCD_Control_port, 0xff);
			if (low_width /= 0) {
			  delay(2*low_width); /* 66 % duty for non negating pins" */
			  outp(LCD_Data_port, 0x00);
			  outp(LCD_Control_port, 0x00);
			  delay(low_width);
			 }
			  else delay(100);
			if (kbhit()!=0) break;
		}
}
#pragma warn +eff

#endif  /* LCDLIB */

